(function() {
'use strict';

var basePath = '/levels/';

angular
.module('Levelselect', ['ngMaterial', 'beamngApi', 'MenuServices'])
.config(function($routeProvider, dashMenuProvider) {
  $routeProvider
    .when('/levelselect', {
      templateUrl: 'modules/levelselect/levelselect.html',
      controller: 'LevelselectController'
    })
    .when('/levelselect/:query', {
      templateUrl: 'modules/levelselect/levelselect.html',
      controller: 'LevelselectController'
    });
//  dashMenuProvider.addMenu({hash: '#/levelselect', title: 'Levels', icon: 'terrain', order: 40});
})
.controller('LevelselectController',  function($scope, $http, $location, $routeParams, $interval, bngApi, filterFilter) {
  //console.log("getLevelList")
  bngApi.engineLua('getLevelList()', function(res) {
    $scope.levels = [];
    for (var i = 0; i < res.length; i++) {
      var levelName = res[i];
      getLevelInfo(levelName);
    }
  });

  if ($routeParams.query) {
    $scope.query = $routeParams.query;
  }

  // TODO: select the first entry of the filtered list always
  var selectedTimer = 0;//Timer to make previews one by one in a certain duration
  var getLevelInfo = function(levelPath) {
	var levelName = levelPath.substring(levelPath.indexOf("/") + 1);
	var levelFolder = levelPath.substring(0, levelPath.indexOf("/"));
    $http({method: 'JSON', url: basePath + levelFolder + '/info.json'})
      .success(function(data, status, headers, config) {
        data.path = basePath + levelFolder + '/';
        data.levelName = levelName;
        if (data.previews && data.previews.length > 0) {
          data.preview = data.previews[0];
          data.preImgIndex = 0;
        }
        $scope.levels.push(data);
        if (!$scope.selected) {
          $scope.selectLevel(data);
        }
      })
      .error(function(data, status) {

        if (status == 404) {
          $scope.levels.push({
            title: levelName,
            description: 'WARNING: no info.json available for this level',
            path: basePath + levelFolder + '/',
            previews: [levelName + '_preview.png'],
            preview: levelName + '_preview.png',
            preImgIndex:0,
            levelName: levelName
          });
        } else {
          console.log('unable to parse json: ' + levelName);
        }
      })
      ;
  };
  var leveltmp;
  function changePreview() {
    $scope.selected.preImgIndex++;
    if ($scope.selected.preImgIndex > $scope.selected.previews.length - 1) {
      $scope.selected.preImgIndex = 0;
    }
    $scope.selected.preview = $scope.selected.previews[$scope.selected.preImgIndex];
  }
  $scope.selectLevel = function(level) {
    $interval.cancel(selectedTimer);
    $scope.selected = level;
    leveltmp = level;
    if (!$scope.selected.previews) {
      $scope.selected.preview = null;
      return;
    }
    $scope.selected.preview = $scope.selected.previews[$scope.selected.preImgIndex];
    if ($scope.selected.previews.length > 1) {
      selectedTimer = $interval(changePreview, 3000);
    }
  };

  $scope.startLevel = function(level) {
    //$scope.selected = level;
    var ts = 'startLevel("' + level.path.substr(1) + level.levelName + '.mis");';
    //console.log(ts);
    bngApi.sendGameEngine(ts);
    window.location.hash = '#/loading';
  };

  // set up the display
  $scope.selectedMenu = '#/levelselect';
  $scope.navClass = 'contentNavLevelselect';
  $scope.useThemeBackground = true;
  $scope.$parent.showDashboard = true;
  // $scope.tileSize = 4;

  // tile clicking: single and double clicking
  $scope.clickTile = function(level) {
    var maxDoubleClickDelayMS = 300; // allow 300 ms for double clicking
    var timestamp = new Date().getTime();
    if (this.lastTileClick !== undefined && timestamp - this.lastTileClick < maxDoubleClickDelayMS) {
      // double click
      $scope.startLevel(level);
    }
    this.lastTileClick = timestamp;
    // boring single click only
    $scope.selectLevel(level);
  };
  
  
  $scope.backToTop = function () {
    $location.path('/toplevelselect');
  };
  
  
});

})();
